#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <argp.h>
#include <signal.h>
#include <execinfo.h>

#include "SDL.h"
#include "SDL_image.h"
#include "util.h"
#include "draw.h"
#include "text.h"

//#define DELAY (1000/FPS)


SDL_Surface *Screen = NULL;
Draw *draw = NULL;
Text *text = NULL;

const SDL_Color grey = {180,180,180};
bool paused = false;
bool Fullscreen = false;
int w=600, h=200;
int cw=0, ch=0;
float cscale = 1.0;

void setvidmode() {
	Uint32 flags = SDL_SWSURFACE | SDL_RESIZABLE | SDL_DOUBLEBUF;
	int mw=w, mh=h;
	if (Fullscreen) {
	    flags |= SDL_FULLSCREEN;
	    mw = 800;
	    mh = 480;
	}
	if (mw<100) mw=100;
	if (mh<50) mh=50;
//	printf("Setting video mode to %s (%ix%i)...\n",(Fullscreen?"full screen":"windowed"),mw,mh);
	if (cw != mw || ch != mh) {
	  Screen = SDL_SetVideoMode(mw, mh, 32, flags);
	  if ( Screen == NULL )
		throw Exiter(SDL_GetError());
	  draw->screen = Screen;
	  if (Fullscreen) {
	    SDL_ShowCursor(SDL_DISABLE);
	  } else {
	    SDL_ShowCursor(SDL_ENABLE);
	  }
	  cw = mw;
	  ch = mh;
	}
	float scale=(mw+200)/1000.0;
	if (cscale != scale) {
	  text->resize(scale);
	  cscale = scale;
	}
//	printf("Done setting video mode\n");
}


void do_key(int k, int down) {
   if (down) {
    switch (k) {
     case SDLK_SPACE: paused = !paused; break;
     case SDLK_q: throw Exiter("Pressed Q"); break;
     case SDLK_ESCAPE: throw Exiter("Pressed ESC"); break;
     case SDLK_f: Fullscreen = !Fullscreen; setvidmode(); break;
    }
   }
}

void mainloop() {
    SDL_Event event;
    char word[200];
    float pos;
    int delay;

    while(1) {
      while(SDL_PollEvent(&event)) {
	switch (event.type) {
		case SDL_KEYDOWN:
			do_key(event.key.keysym.sym,1);
			break;
		case SDL_KEYUP:
			do_key(event.key.keysym.sym,0);
			break;
		case SDL_VIDEORESIZE:
			w=event.resize.w;
			h=event.resize.h;
			setvidmode();
			break;
		case SDL_QUIT:
			throw Exiter("Close button of window clicked");
//		default: printf("Unknown event: %i\n",event.type);
	}
      }
      if (!paused) {
      pos = -100;
      scanf("%f %i %s\n",&pos,&delay,word);
      rmnl(word);
      if (pos == -100) throw Exiter("End");
      pos += 0.5;
      }
      draw->draw(word,pos);
      if (paused) text->drawtext("Paused",Screen,Screen->w,Screen->h, grey,text->normal,1000,1000);
      SDL_Flip(Screen);
      SDL_Delay(delay);
    }
}
static const bool kUseExceptionHandler = false;
//static const bool kUseExceptionHandler = true;

// thanks to Steven Craft for this snippet
void handler(int sig) {
    void *array[ 16 ];
    size_t size;
    // get void*'s for all entries on the stack
    size = backtrace(array, sizeof( array ) );
    // print out all the frames to stderr
    fprintf(stderr, "Error: signal %d:\n", sig);
    backtrace_symbols_fd( array, size, 2 );
    SDL_Quit();
    exit( 1 );
}


int main(int argc, char **argv) {

	if ( kUseExceptionHandler ) {
		signal(SIGSEGV, handler);
		signal(SIGBUS, handler);
	}



	printf("Initializing SDL: VIDEO...");
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
		fprintf(stderr, "Error initializing SDL: %s\n", SDL_GetError());
		return 1;
	}
	int rv = 0;
	try {
		printf("OK.\n");
		
		SDL_WM_SetCaption("SpeedRead", NULL);
		//Screen = SDL_SetVideoMode(800, 480, 32, flags);
		printf("init text\n");
		text = new Text();
		printf("init draw\n");
		draw = new Draw(0,text);
		printf("init video\n");
		setvidmode();
		if ( Screen == NULL )
			throw Exiter(SDL_GetError());
		else
			printf("OK\n");

//		SDL_Surface *icon=IMG_Load("icon.png");
//		SDL_WM_SetIcon(icon, NULL);


		printf("begin\n");
		mainloop();
	}
	catch(Exiter &e)
	{
		printf("\nExiting: %s\n", e.message.c_str());
		rv = e.value;
	}
	
	
	SDL_Quit();
	return rv;
}

