#include "text.h"
const SDL_Color black = {0,0,0};

void Text::resize(float f) {
//	printf("Init fonts at scale %f\n",f);
	if (huge) {TTF_CloseFont(huge); huge=NULL;}
	if (normal) {TTF_CloseFont(normal); normal=NULL;}
	if (small) {TTF_CloseFont(small); small=NULL;}
	if (!huge) huge=TTF_OpenFont("/usr/share/fonts/truetype/DejaVuSerif.ttf",100*f);
	if (!normal) normal=TTF_OpenFont("/usr/share/fonts/truetype/LiberationSans-Regular.ttf",36*f);
	if (!small) small=TTF_OpenFont("/usr/share/fonts/truetype/LiberationSans-Regular.ttf",28*f);
	if (!huge) throw Exiter("Could not load font...");
	reset_cache();
}
void Text::reset_cache() {
	for (int i=0; i<CACHE_SIZE; i++) {
	    cache_string[i][0]=0;
	}
}


Text::Text() {
	if(TTF_Init()) throw Exiter(TTF_GetError());
	huge=normal=small=NULL;
	resize(1.0);
	for (int i=0; i<CACHE_SIZE; i++) {
	    cache[i]=NULL;
	    cache_string[i][0]=0;
	}
}

int Text::drawtext_cache(const char * t, SDL_Surface *surf, int x, int y, SDL_Color c, TTF_Font *f, int halign, int valign, int cid) {
    if (cid<0 || cid>CACHE_SIZE) return -1;
    if (strcmp(t,cache_string[cid])) {
      if (cache[cid]) SDL_FreeSurface(cache[cid]);
      cache[cid]=rendertext(t,c,f);
      strcpy(cache_string[cid],t);
//      printf("rendering string:'%s' @ %i\n",cache_string[cid],cid);
    }
    return drawrenderedtext(cache[cid], surf, x, y, halign, valign);
}

int Text::drawtext(const char * t, SDL_Surface *surf, int x, int y, SDL_Color c, TTF_Font *f, int halign, int valign) {
//    if (y < 30) return -1;
    SDL_Surface *text;
    text = rendertext(t, c, f);
    int rx = drawrenderedtext(text, surf, x, y, halign, valign);
    SDL_FreeSurface(text);
    return rx;
}
void Text::drawtext(const char * t, SDL_Surface *surf, int x, int y, SDL_Color c, TTF_Font *f) {
    drawtext(t, surf, x, y, c, f, 0, 0);
}

SDL_Surface* Text::rendertext(const char * t, SDL_Color c, TTF_Font *f) {
//    return TTF_RenderText_Shaded(f, t, c, black);
    return TTF_RenderText_Blended(f, t, c);
}
inline int Text::drawrenderedtext(SDL_Surface *text, SDL_Surface *surf, int x, int y, int halign, int valign) {
    if (!text) throw Exiter("No text surface!");
    SDL_Rect re;
    re.x = x - (text->w * halign / 1000);
    re.y = y - (text->h * valign / 1000);

//    re.w = text->w;
//    re.h = text->h;
//    printf("%ix%i  at %i,%i\n",text->w,text->h,re.x,re.y);
    SDL_BlitSurface(text,NULL,surf,&re);
//    printf("ok\n");
    return re.x;
}



